class Cancelable {
  bool _isCanceled = false;

  bool get isCanceled => _isCanceled;

  cancel() {
    _isCanceled = true;
  }
}

mixin CancelableObject {
  final cancelable = Cancelable();

  R Function(A) doIf<R, A>(R Function(A) action, [doOnCancel()]) {
    if (action != null) {
      return (A a) {
        if (!cancelable.isCanceled) {
          return action(a);
        } else {
          if (doOnCancel != null) doOnCancel();
          return null;
        }
      };
    } else {
      return (_) {
        if (cancelable.isCanceled) {
          if (doOnCancel != null) doOnCancel();
        }
        return null;
      };
    }
  }

  R Function() doneIf<R, A>(R Function() action, [doOnCancel()]) {
    if (action != null) {
      return () {
        if (!cancelable.isCanceled) {
          return action();
        } else {
          if (doOnCancel != null) doOnCancel();
          return null;
        }
      };
    } else {
      return () {
        if (cancelable.isCanceled) {
          if (doOnCancel != null) doOnCancel();
        }
        return null;
      };
    }
  }
}
