import 'dart:async';

import 'cancelable.dart';

class CancelableFuture<T> with CancelableObject implements Future<T> {
  final Future<T> wrapped;

  CancelableFuture(this.wrapped);

  @override
  Stream<T> asStream() => wrapped.asStream();

  @override
  Future<T> catchError(Function onError, {bool test(Object error)}) =>
      wrapped.catchError(doIf(onError), test: doIf(test));

  @override
  Future<S> then<S>(FutureOr<S> onValue(T value), {Function onError}) =>
      wrapped.then(doIf(onValue), onError: doIf(onError));

  @override
  Future<T> timeout(Duration timeLimit, {FutureOr<T> onTimeout()}) =>
      wrapped.timeout(timeLimit, onTimeout: doneIf(onTimeout));

  @override
  Future<T> whenComplete(void action()) => wrapped.whenComplete(() {
        doneIf(action)();
        cancelable.cancel();
      });
}
