import 'dart:async';

import 'cancelable.dart';

class CancelableStream<T> extends Stream<T> with CancelableObject {
  final Stream<T> wrapped;

  CancelableStream(this.wrapped);

  @override
  StreamSubscription<T> listen(void Function(T event) onData,
      {Function onError, void Function() onDone, bool cancelOnError}) {
    StreamSubscription<T> sub;
    sub = wrapped.listen(
      doIf(onData, () => sub.cancel),
      onError: doIf(onError, () => sub.cancel),
      onDone: () {
        doneIf(onDone, () => sub.cancel)();
        cancelable.cancel();
      },
      cancelOnError: cancelOnError,
    );
    return sub;
  }
}
